/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include "fitsio.h"
#include "imaging_fdata.h"
#include "flag.h"

/*
    Usage::
    oirAverageVisImage infile flagFile outFile 
*/
void Usage() {
   printf("   oirAverageVisImage inFile flagFile outFile\n");
   printf("   inFile = file name of output of oirRotateGroupDelay\n");
   printf("   flagFile = file name of output of oirAutoFlag\n");
   printf("   outFile = output file name containing averaged correlated flux\n");
   return;
}
int main (int argc, char** argv) {

   int  i, j, ierr;
   long nRow, iRow;
   int  nx, ny;
   int  nregion, iregion;
   long npix;

   char outFileName[130];
   char *bang="!";

   fitsfile *inFile;
   fitsfile *flagFile;
   fitsfile *outFile = NULL;
   imaging_fdata_table_def *idata;
   imaging_fdata_table_def *odata;
   imaging_detector_table_def *outDetectorDef;
   imaging_fdata_row *inData;
   imaging_fdata_row *outData;

   flag_table_def *fTable;
   flag_table_row *fRow;
   int iFlag;
   long  nFlag;
   double* flagTimes;

   int nGood, good;
   float totalExposure;
   double visTime;


   ierr = 0;
/*
      open input file 
*/
   if (argc < 3) {
      Usage();
      return 1;
   }
   printf("      ****** oirAverageVisImage ******\n");
   fits_open_file (&inFile, argv[1], READONLY, &ierr);
   if (ierr != 0) {
      printf("fits openfile of %s gave error %i\n",argv[1],
         ierr);
      printf("%s \n",strerror(errno));
      return 1;
   }
/*
      create output file and copy header
*/
   strcpy(outFileName, bang);
   strcat(outFileName, argv[3]);

   fits_create_file (&outFile, outFileName, &ierr);
   if (ierr !=0) {
      fprintf (stderr, "error opening fits output %s %i\n",argv[3], ierr);
      printf("%s \n",strerror(errno));
      return 1;
   }
   i = fits_copy_header (inFile, outFile, &ierr);

/*
      get detector info
*/

   outDetectorDef = read_imaging_detector_table(inFile, &ierr);
   idata          = open_imaging_fdata_table(inFile, &ierr);
   inData         = make_imaging_fdata_row(idata);

/*     
   get row size (pseudo-complex)
*/
   nx   = outDetectorDef->naxis[0][0];
   ny   = outDetectorDef->naxis[0][1];
   npix = nx* (long)ny;
   nregion = outDetectorDef->nregion;
   i    = fits_get_num_rows (inFile,  &nRow, &ierr); 
/*
   set up output table definitions
*/
   odata = make_imaging_fdata_table_def (outFile, outDetectorDef, 1, &ierr);
   if (ierr !=0) {
      fprintf (stderr, "error creating data definition\n");
      return 1;
   }
   outData = make_imaging_fdata_row(odata);

/*
      read in flag data and expand
*/
   fits_open_file (&flagFile, argv[2], READONLY, &ierr);
   if (ierr != 0) {
      printf("fits openfile of flagfile %s gave error %i\n",argv[2],
         ierr);
      printf("%s \n",strerror(errno));
      return 1;
   }
   fTable  = open_flag_table(flagFile, &ierr);
   if (ierr != 0) {
      printf("create flag table gave error %i\n", ierr);
      return 1;
   }
   fRow = make_flag_table_row(fTable);
   i = fits_get_num_rows (flagFile,  &nFlag, &ierr); 
   flagTimes = (double *)malloc(nFlag * 2 * sizeof(double));
   for (iRow =0; iRow <nFlag; iRow++) {
      i=read_flag_table(flagFile, fTable, fRow, iRow+1, &ierr);
      flagTimes[2*iRow]   = fRow->timeRange[0];
      flagTimes[2*iRow+1] = fRow->timeRange[1];
   }
   i = close_flag_table(flagFile, fTable, &ierr);
   kill_flag_table_row(fRow);


   nGood = 0;
   totalExposure = 0.;
/*
         read through data once and set up flag table
         check against flag times; it would be faster if we sorted 
         times first; average visibilities coherently
*/
   for (iRow = 1; iRow <= nRow; iRow++) {
      i = read_imaging_fdata_table(inFile, idata, inData, iRow, &ierr);
      good = 1;
      visTime = inData->time;
      if (isnan(visTime)) good = 0; else 
         for (iFlag =0; iFlag<nFlag && (good == 1); iFlag++) 
            if (visTime >= flagTimes[2*iFlag] && (visTime <= flagTimes[2*iFlag+1])) good=0;
      if (good == 1) {
         nGood = nGood + 1;
         totalExposure = totalExposure + inData->exp;
         for (iregion=0;iregion<nregion;iregion++) 
            for (i=0;i<npix;i++) outData->data[iregion][i] 
               = outData->data[iregion][i]+inData->data[iregion][i];
      }
   }
   if (nGood > 0) {
      for (iregion=0;iregion<nregion;iregion++) 
         for (i=0;i<npix;i++) outData->data[iregion][i] = 
            outData->data[iregion][i]/nGood;
      printf("   Averaged %i so-called good frames.\n", nGood);
   }  else {
         printf("No unflagged data in input\n");
         return 0;
   }
/*
      write output
*/
   i = write_imaging_fdata_table(outFile, odata, outData, 1, &ierr);

/*
      shut down and shut up
*/
   i = close_imaging_fdata_table(inFile, idata, &ierr);
   i = close_imaging_fdata_table(outFile, odata, &ierr);
   kill_imaging_fdata_row(inData);
   kill_imaging_fdata_row(outData);
   fits_close_file(outFile, &ierr);
   free(flagTimes);
   return 0;
}
